# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'macroToolTemplate.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(803, 883)
        self.gridLayout_2 = QtWidgets.QGridLayout(Form)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.splitter = QtWidgets.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.widget_macro_designer = QtWidgets.QWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget_macro_designer.sizePolicy().hasHeightForWidth())
        self.widget_macro_designer.setSizePolicy(sizePolicy)
        self.widget_macro_designer.setMaximumSize(QtCore.QSize(450, 16777215))
        self.widget_macro_designer.setObjectName("widget_macro_designer")
        self.gridLayout = QtWidgets.QGridLayout(self.widget_macro_designer)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.layout_template_select = QtWidgets.QFormLayout()
        self.layout_template_select.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.layout_template_select.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.layout_template_select.setObjectName("layout_template_select")
        self.label_template = QtWidgets.QLabel(self.widget_macro_designer)
        self.label_template.setObjectName("label_template")
        self.layout_template_select.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_template)
        self.combobox_template = QtWidgets.QComboBox(self.widget_macro_designer)
        self.combobox_template.setObjectName("combobox_template")
        self.combobox_template.addItem("")
        self.layout_template_select.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.combobox_template)
        self.gridLayout.addLayout(self.layout_template_select, 1, 0, 1, 1)
        self.label_new_macro = QtWidgets.QLabel(self.widget_macro_designer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_new_macro.sizePolicy().hasHeightForWidth())
        self.label_new_macro.setSizePolicy(sizePolicy)
        self.label_new_macro.setStyleSheet("font-weight:bold;background:#f6b442;padding:5")
        self.label_new_macro.setObjectName("label_new_macro")
        self.gridLayout.addWidget(self.label_new_macro, 0, 0, 1, 1)
        self.widget_macro_components = QtWidgets.QStackedWidget(self.widget_macro_designer)
        self.widget_macro_components.setObjectName("widget_macro_components")
        self.page_macro_components = QtWidgets.QWidget()
        self.page_macro_components.setObjectName("page_macro_components")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.page_macro_components)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.button_start_macro = QtWidgets.QPushButton(self.page_macro_components)
        self.button_start_macro.setStatusTip("")
        self.button_start_macro.setStyleSheet("QPushButton { background-color: #68d532 }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/play.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_start_macro.setIcon(icon)
        self.button_start_macro.setIconSize(QtCore.QSize(16, 16))
        self.button_start_macro.setObjectName("button_start_macro")
        self.horizontalLayout.addWidget(self.button_start_macro)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.button_remove = QtWidgets.QPushButton(self.page_macro_components)
        self.button_remove.setStyleSheet("QPushButton { background-color: #ff4e00 }\\\\nQPushButton:pressed { background-color: #ff4e00 }\\\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\\nQPushButton:disabled{ background:silver;color:grey }")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_remove.setIcon(icon1)
        self.button_remove.setIconSize(QtCore.QSize(16, 16))
        self.button_remove.setObjectName("button_remove")
        self.horizontalLayout.addWidget(self.button_remove)
        self.gridLayout_4.addLayout(self.horizontalLayout, 3, 0, 1, 1)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.label_2 = QtWidgets.QLabel(self.page_macro_components)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.input_comment = QtWidgets.QLineEdit(self.page_macro_components)
        self.input_comment.setObjectName("input_comment")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.input_comment)
        self.label_4 = QtWidgets.QLabel(self.page_macro_components)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.input_repeat_macro = QtWidgets.QDoubleSpinBox(self.page_macro_components)
        self.input_repeat_macro.setDecimals(0)
        self.input_repeat_macro.setMaximum(2140000000.0)
        self.input_repeat_macro.setObjectName("input_repeat_macro")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.input_repeat_macro)
        self.gridLayout_4.addLayout(self.formLayout, 1, 0, 1, 1)
        self.scroll_area_macro = QtWidgets.QScrollArea(self.page_macro_components)
        self.scroll_area_macro.setWidgetResizable(True)
        self.scroll_area_macro.setObjectName("scroll_area_macro")
        self.widget_macro_list = QtWidgets.QWidget()
        self.widget_macro_list.setGeometry(QtCore.QRect(0, 0, 448, 383))
        self.widget_macro_list.setObjectName("widget_macro_list")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget_macro_list)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scroll_area_macro.setWidget(self.widget_macro_list)
        self.gridLayout_4.addWidget(self.scroll_area_macro, 2, 0, 1, 1)
        self.widget_macro_components.addWidget(self.page_macro_components)
        self.page_template_designer = QtWidgets.QWidget()
        self.page_template_designer.setObjectName("page_template_designer")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.page_template_designer)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.scroll_area_custom_template = QtWidgets.QScrollArea(self.page_template_designer)
        self.scroll_area_custom_template.setWidgetResizable(True)
        self.scroll_area_custom_template.setObjectName("scroll_area_custom_template")
        self.widget_custom_template = QtWidgets.QWidget()
        self.widget_custom_template.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.widget_custom_template.setObjectName("widget_custom_template")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.widget_custom_template)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.scroll_area_custom_template.setWidget(self.widget_custom_template)
        self.gridLayout_5.addWidget(self.scroll_area_custom_template, 0, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.button_reset_template = QtWidgets.QPushButton(self.page_template_designer)
        self.button_reset_template.setStatusTip("")
        self.button_reset_template.setStyleSheet("QPushButton { background-color: #ff4e00 }\\\\nQPushButton:pressed { background-color: #ff4e00 }\\\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\\nQPushButton:disabled{ background:silver;color:grey }")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/icons/reset_bw.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_reset_template.setIcon(icon2)
        self.button_reset_template.setIconSize(QtCore.QSize(16, 16))
        self.button_reset_template.setObjectName("button_reset_template")
        self.horizontalLayout_5.addWidget(self.button_reset_template)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem1)
        self.button_save_template = QtWidgets.QPushButton(self.page_template_designer)
        self.button_save_template.setStyleSheet("QPushButton { background-color: #00b4ff }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/icons/save_bw.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_save_template.setIcon(icon3)
        self.button_save_template.setIconSize(QtCore.QSize(16, 16))
        self.button_save_template.setObjectName("button_save_template")
        self.horizontalLayout_5.addWidget(self.button_save_template)
        self.gridLayout_5.addLayout(self.horizontalLayout_5, 5, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.button_add_iterative_step = QtWidgets.QPushButton(self.page_template_designer)
        self.button_add_iterative_step.setStatusTip("")
        self.button_add_iterative_step.setStyleSheet("QPushButton { background-color: #68d532 }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        self.button_add_iterative_step.setIconSize(QtCore.QSize(16, 16))
        self.button_add_iterative_step.setObjectName("button_add_iterative_step")
        self.horizontalLayout_4.addWidget(self.button_add_iterative_step)
        self.button_add_action_step = QtWidgets.QPushButton(self.page_template_designer)
        self.button_add_action_step.setStatusTip("")
        self.button_add_action_step.setStyleSheet("QPushButton { background-color: #68d532 }\\nQPushButton:pressed { background-color: #28a900 }\\nQPushButton:!enabled{background:silver;}QPushButton:checked{background:red;}\n"
"QPushButton:disabled{ background:silver;color:grey }")
        self.button_add_action_step.setIconSize(QtCore.QSize(16, 16))
        self.button_add_action_step.setObjectName("button_add_action_step")
        self.horizontalLayout_4.addWidget(self.button_add_action_step)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.gridLayout_5.addLayout(self.horizontalLayout_4, 2, 0, 1, 1)
        self.label_measurement_group = QtWidgets.QLabel(self.page_template_designer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_measurement_group.sizePolicy().hasHeightForWidth())
        self.label_measurement_group.setSizePolicy(sizePolicy)
        self.label_measurement_group.setStyleSheet("font-weight:bold;background:#f6b442;padding:5")
        self.label_measurement_group.setObjectName("label_measurement_group")
        self.gridLayout_5.addWidget(self.label_measurement_group, 3, 0, 1, 1)
        self.list_measurement_groups = QtWidgets.QListWidget(self.page_template_designer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_measurement_groups.sizePolicy().hasHeightForWidth())
        self.list_measurement_groups.setSizePolicy(sizePolicy)
        self.list_measurement_groups.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.list_measurement_groups.setObjectName("list_measurement_groups")
        self.gridLayout_5.addWidget(self.list_measurement_groups, 4, 0, 1, 1)
        self.widget_macro_components.addWidget(self.page_template_designer)
        self.gridLayout.addWidget(self.widget_macro_components, 3, 0, 1, 1)
        self.tab_widget_macro_list = QtWidgets.QTabWidget(self.splitter)
        self.tab_widget_macro_list.setAutoFillBackground(False)
        self.tab_widget_macro_list.setStyleSheet("")
        self.tab_widget_macro_list.setDocumentMode(True)
        self.tab_widget_macro_list.setTabsClosable(True)
        self.tab_widget_macro_list.setObjectName("tab_widget_macro_list")
        self.gridLayout_2.addWidget(self.splitter, 0, 0, 1, 1)

        self.retranslateUi(Form)
        self.widget_macro_components.setCurrentIndex(0)
        self.combobox_template.currentTextChanged['QString'].connect(Form.template_selected)
        self.button_add_iterative_step.clicked.connect(Form.add_iterative_step)
        self.button_add_action_step.clicked.connect(Form.add_action_step)
        self.button_reset_template.clicked.connect(Form.reset_template_input)
        self.button_save_template.clicked.connect(Form.export_template)
        self.button_remove.clicked.connect(Form.remove_template)
        self.button_start_macro.clicked.connect(Form.start_macro)
        self.tab_widget_macro_list.tabCloseRequested['int'].connect(Form.close_macro_tab)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Scanning tool"))
        self.label_template.setText(_translate("Form", "Template:"))
        self.combobox_template.setItemText(0, _translate("Form", "Custom"))
        self.label_new_macro.setText(_translate("Form", "Create new macro"))
        self.button_start_macro.setToolTip(_translate("Form", "Start scan"))
        self.button_start_macro.setText(_translate("Form", "Start"))
        self.button_remove.setToolTip(_translate("Form", "Remove selected template"))
        self.button_remove.setText(_translate("Form", "Remove"))
        self.label_2.setText(_translate("Form", "Comment:"))
        self.label_4.setText(_translate("Form", "Repeat:"))
        self.button_reset_template.setToolTip(_translate("Form", "Reset template"))
        self.button_reset_template.setText(_translate("Form", "Reset"))
        self.button_save_template.setToolTip(_translate("Form", "Save template"))
        self.button_save_template.setText(_translate("Form", "Save"))
        self.button_add_iterative_step.setToolTip(_translate("Form", "Add iterative step"))
        self.button_add_iterative_step.setText(_translate("Form", "+ Iterative step"))
        self.button_add_action_step.setToolTip(_translate("Form", "Add action step"))
        self.button_add_action_step.setText(_translate("Form", "+ Action step"))
        self.label_measurement_group.setText(_translate("Form", "Select measurement group(s)"))
        self.list_measurement_groups.setSortingEnabled(True)
from kamzik3 import resource_kamzik3_rc
