# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'deviceDebugTemplate.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(724, 222)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.widgets_stack = QtWidgets.QStackedWidget(Form)
        self.widgets_stack.setObjectName("widgets_stack")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.page)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.widget_attributes = QtWidgets.QWidget(self.page)
        self.widget_attributes.setObjectName("widget_attributes")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget_attributes)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tab_attribute_groups = QtWidgets.QTabWidget(self.widget_attributes)
        self.tab_attribute_groups.setObjectName("tab_attribute_groups")
        self.verticalLayout.addWidget(self.tab_attribute_groups)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(6)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.input_attribute_filter = QtWidgets.QLineEdit(self.widget_attributes)
        self.input_attribute_filter.setInputMask("")
        self.input_attribute_filter.setObjectName("input_attribute_filter")
        self.horizontalLayout_2.addWidget(self.input_attribute_filter)
        self.pushButton_2 = QtWidgets.QPushButton(self.widget_attributes)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/search.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_2.setIcon(icon)
        self.pushButton_2.setObjectName("pushButton_2")
        self.horizontalLayout_2.addWidget(self.pushButton_2)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout_2.addWidget(self.widget_attributes, 0, 0, 1, 1)
        self.widgets_stack.addWidget(self.page)
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setObjectName("page_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.page_2)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.widget_configuration = QtWidgets.QWidget(self.page_2)
        self.widget_configuration.setObjectName("widget_configuration")
        self.gridLayout_3.addWidget(self.widget_configuration, 0, 0, 1, 1)
        self.widgets_stack.addWidget(self.page_2)
        self.page_3 = QtWidgets.QWidget()
        self.page_3.setObjectName("page_3")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.page_3)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.widget_terminal = QtWidgets.QWidget(self.page_3)
        self.widget_terminal.setObjectName("widget_terminal")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.widget_terminal)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_terminator = QtWidgets.QLabel(self.widget_terminal)
        self.label_terminator.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_terminator.setObjectName("label_terminator")
        self.gridLayout_5.addWidget(self.label_terminator, 0, 2, 1, 1)
        self.label_command = QtWidgets.QLabel(self.widget_terminal)
        self.label_command.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_command.setObjectName("label_command")
        self.gridLayout_5.addWidget(self.label_command, 0, 0, 1, 1)
        self.input_command = QtWidgets.QLineEdit(self.widget_terminal)
        self.input_command.setObjectName("input_command")
        self.gridLayout_5.addWidget(self.input_command, 0, 1, 1, 1)
        self.button_send = QtWidgets.QPushButton(self.widget_terminal)
        self.button_send.setObjectName("button_send")
        self.gridLayout_5.addWidget(self.button_send, 0, 4, 1, 1)
        self.output_commands = QtWidgets.QTextBrowser(self.widget_terminal)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.output_commands.sizePolicy().hasHeightForWidth())
        self.output_commands.setSizePolicy(sizePolicy)
        self.output_commands.setMinimumSize(QtCore.QSize(0, 50))
        self.output_commands.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.output_commands.setObjectName("output_commands")
        self.gridLayout_5.addWidget(self.output_commands, 1, 0, 1, 5)
        self.comboBox_terminator = QtWidgets.QComboBox(self.widget_terminal)
        self.comboBox_terminator.setObjectName("comboBox_terminator")
        self.comboBox_terminator.addItem("")
        self.comboBox_terminator.addItem("")
        self.comboBox_terminator.addItem("")
        self.comboBox_terminator.addItem("")
        self.comboBox_terminator.addItem("")
        self.gridLayout_5.addWidget(self.comboBox_terminator, 0, 3, 1, 1)
        self.gridLayout_4.addWidget(self.widget_terminal, 0, 0, 1, 1)
        self.widgets_stack.addWidget(self.page_3)
        self.page_4 = QtWidgets.QWidget()
        self.page_4.setObjectName("page_4")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.page_4)
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_6.setVerticalSpacing(6)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.scroll_area_methods = QtWidgets.QScrollArea(self.page_4)
        self.scroll_area_methods.setWidgetResizable(True)
        self.scroll_area_methods.setObjectName("scroll_area_methods")
        self.widget_grid_for_methods = QtWidgets.QWidget()
        self.widget_grid_for_methods.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.widget_grid_for_methods.setObjectName("widget_grid_for_methods")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.widget_grid_for_methods)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.scroll_area_methods.setWidget(self.widget_grid_for_methods)
        self.gridLayout_6.addWidget(self.scroll_area_methods, 0, 0, 1, 1)
        self.output_methods = QtWidgets.QPlainTextEdit(self.page_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.output_methods.sizePolicy().hasHeightForWidth())
        self.output_methods.setSizePolicy(sizePolicy)
        self.output_methods.setMaximumSize(QtCore.QSize(16777215, 50))
        self.output_methods.setReadOnly(True)
        self.output_methods.setOverwriteMode(True)
        self.output_methods.setObjectName("output_methods")
        self.gridLayout_6.addWidget(self.output_methods, 1, 0, 1, 1)
        self.widgets_stack.addWidget(self.page_4)
        self.gridLayout.addWidget(self.widgets_stack, 2, 0, 1, 1)
        self.label_device_id = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_device_id.sizePolicy().hasHeightForWidth())
        self.label_device_id.setSizePolicy(sizePolicy)
        self.label_device_id.setMinimumSize(QtCore.QSize(0, 35))
        font = QtGui.QFont()
        font.setPointSize(23)
        font.setBold(True)
        font.setWeight(75)
        self.label_device_id.setFont(font)
        self.label_device_id.setStyleSheet("QLabel{background:#68d532;}")
        self.label_device_id.setAlignment(QtCore.Qt.AlignCenter)
        self.label_device_id.setObjectName("label_device_id")
        self.gridLayout.addWidget(self.label_device_id, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.button_attributes = QtWidgets.QPushButton(Form)
        self.button_attributes.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/settings.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_attributes.setIcon(icon1)
        self.button_attributes.setIconSize(QtCore.QSize(20, 20))
        self.button_attributes.setCheckable(True)
        self.button_attributes.setChecked(True)
        self.button_attributes.setAutoExclusive(True)
        self.button_attributes.setObjectName("button_attributes")
        self.horizontalLayout.addWidget(self.button_attributes)
        self.button_attributes_2 = QtWidgets.QPushButton(Form)
        self.button_attributes_2.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        self.button_attributes_2.setIcon(icon1)
        self.button_attributes_2.setIconSize(QtCore.QSize(20, 20))
        self.button_attributes_2.setCheckable(True)
        self.button_attributes_2.setChecked(False)
        self.button_attributes_2.setAutoExclusive(True)
        self.button_attributes_2.setObjectName("button_attributes_2")
        self.horizontalLayout.addWidget(self.button_attributes_2)
        self.button_configuration = QtWidgets.QPushButton(Form)
        self.button_configuration.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/icons/configFile.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_configuration.setIcon(icon2)
        self.button_configuration.setIconSize(QtCore.QSize(20, 20))
        self.button_configuration.setCheckable(True)
        self.button_configuration.setAutoExclusive(True)
        self.button_configuration.setObjectName("button_configuration")
        self.horizontalLayout.addWidget(self.button_configuration)
        self.button_terminal = QtWidgets.QPushButton(Form)
        self.button_terminal.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/icons/terminal.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_terminal.setIcon(icon3)
        self.button_terminal.setIconSize(QtCore.QSize(20, 20))
        self.button_terminal.setCheckable(True)
        self.button_terminal.setAutoExclusive(True)
        self.button_terminal.setObjectName("button_terminal")
        self.horizontalLayout.addWidget(self.button_terminal)
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.button_enable = QtWidgets.QPushButton(Form)
        self.button_enable.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/icons/play.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_enable.setIcon(icon4)
        self.button_enable.setIconSize(QtCore.QSize(20, 20))
        self.button_enable.setCheckable(False)
        self.button_enable.setAutoExclusive(False)
        self.button_enable.setObjectName("button_enable")
        self.horizontalLayout.addWidget(self.button_enable)
        self.button_disable = QtWidgets.QPushButton(Form)
        self.button_disable.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/icons/icons/cancel.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_disable.setIcon(icon5)
        self.button_disable.setIconSize(QtCore.QSize(20, 20))
        self.button_disable.setCheckable(False)
        self.button_disable.setAutoExclusive(False)
        self.button_disable.setObjectName("button_disable")
        self.horizontalLayout.addWidget(self.button_disable)
        self.button_connect = QtWidgets.QPushButton(Form)
        self.button_connect.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/icons/icons/plug.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_connect.setIcon(icon6)
        self.button_connect.setIconSize(QtCore.QSize(20, 20))
        self.button_connect.setCheckable(False)
        self.button_connect.setAutoExclusive(False)
        self.button_connect.setObjectName("button_connect")
        self.horizontalLayout.addWidget(self.button_connect)
        self.button_disconnect = QtWidgets.QPushButton(Form)
        self.button_disconnect.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/icons/icons/logout.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_disconnect.setIcon(icon7)
        self.button_disconnect.setIconSize(QtCore.QSize(20, 20))
        self.button_disconnect.setCheckable(False)
        self.button_disconnect.setAutoExclusive(False)
        self.button_disconnect.setObjectName("button_disconnect")
        self.horizontalLayout.addWidget(self.button_disconnect)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(Form)
        self.widgets_stack.setCurrentIndex(0)
        self.button_attributes.clicked.connect(Form.show_attributes_widget)
        self.button_configuration.clicked.connect(Form.show_configuration_widget)
        self.button_terminal.clicked.connect(Form.show_terminal_widget)
        self.button_send.clicked.connect(Form.send_command)
        self.input_command.returnPressed.connect(Form.send_command)
        self.button_disconnect.clicked.connect(Form.disconnect_device)
        self.button_connect.clicked.connect(Form.connect_device)
        self.button_disable.clicked.connect(Form.disable_device)
        self.button_enable.clicked.connect(Form.enable_device)
        self.tab_attribute_groups.currentChanged['int'].connect(Form.slot_attribute_group_changed)
        self.button_attributes_2.clicked.connect(Form.show_methods_widget)
        self.comboBox_terminator.activated['QString'].connect(Form.set_command_terminator)
        self.pushButton_2.clicked.connect(Form.filter_attributes)
        self.input_attribute_filter.returnPressed.connect(self.pushButton_2.click)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.input_attribute_filter.setPlaceholderText(_translate("Form", "Filter attributes"))
        self.pushButton_2.setText(_translate("Form", "Filter"))
        self.label_terminator.setText(_translate("Form", "Terminator:"))
        self.label_command.setText(_translate("Form", "Command:"))
        self.button_send.setText(_translate("Form", "Send"))
        self.comboBox_terminator.setItemText(0, _translate("Form", "None"))
        self.comboBox_terminator.setItemText(1, _translate("Form", "\\r\\n"))
        self.comboBox_terminator.setItemText(2, _translate("Form", "\\r"))
        self.comboBox_terminator.setItemText(3, _translate("Form", "\\n"))
        self.comboBox_terminator.setItemText(4, _translate("Form", "\\t"))
        self.label_device_id.setText(_translate("Form", "Device Id"))
        self.button_attributes.setToolTip(_translate("Form", "Show device attributes page"))
        self.button_attributes.setText(_translate("Form", "Attributes"))
        self.button_attributes_2.setToolTip(_translate("Form", "Show device attributes page"))
        self.button_attributes_2.setText(_translate("Form", "Methods"))
        self.button_configuration.setToolTip(_translate("Form", "Show device configuration page"))
        self.button_configuration.setText(_translate("Form", "Configuration"))
        self.button_terminal.setToolTip(_translate("Form", "Show device terminal page"))
        self.button_terminal.setText(_translate("Form", "Terminal"))
        self.button_enable.setToolTip(_translate("Form", "Enable device"))
        self.button_enable.setText(_translate("Form", "Enable"))
        self.button_disable.setToolTip(_translate("Form", "Disable device"))
        self.button_disable.setText(_translate("Form", "Disable"))
        self.button_connect.setToolTip(_translate("Form", "Connect device"))
        self.button_connect.setText(_translate("Form", "Connect"))
        self.button_disconnect.setToolTip(_translate("Form", "Disconnect device"))
        self.button_disconnect.setText(_translate("Form", "Disconnect"))

from kamzik3 import resource_kamzik3_rc
