# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'macroStepTemplate.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(320, 189)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.combobox_step = QtWidgets.QComboBox(Form)
        self.combobox_step.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.combobox_step.setObjectName("combobox_step")
        self.gridLayout.addWidget(self.combobox_step, 3, 1, 1, 1)
        self.widget_settings = QtWidgets.QWidget(Form)
        self.widget_settings.setObjectName("widget_settings")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.widget_settings)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.checkbox_defaults = QtWidgets.QCheckBox(self.widget_settings)
        self.checkbox_defaults.setChecked(False)
        self.checkbox_defaults.setObjectName("checkbox_defaults")
        self.gridLayout_2.addWidget(self.checkbox_defaults, 0, 1, 1, 1)
        self.checkbox_log = QtWidgets.QCheckBox(self.widget_settings)
        self.checkbox_log.setChecked(True)
        self.checkbox_log.setObjectName("checkbox_log")
        self.gridLayout_2.addWidget(self.checkbox_log, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.widget_settings, 1, 1, 1, 1)
        self.label_scanner = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_scanner.sizePolicy().hasHeightForWidth())
        self.label_scanner.setSizePolicy(sizePolicy)
        self.label_scanner.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_scanner.setFont(font)
        self.label_scanner.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_scanner.setObjectName("label_scanner")
        self.gridLayout.addWidget(self.label_scanner, 6, 0, 1, 1)
        self.label_method = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_method.sizePolicy().hasHeightForWidth())
        self.label_method.setSizePolicy(sizePolicy)
        self.label_method.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_method.setFont(font)
        self.label_method.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_method.setObjectName("label_method")
        self.gridLayout.addWidget(self.label_method, 5, 0, 1, 1)
        self.combobox_device = QtWidgets.QComboBox(Form)
        self.combobox_device.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.combobox_device.setObjectName("combobox_device")
        self.gridLayout.addWidget(self.combobox_device, 2, 1, 1, 1)
        self.label_step = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_step.sizePolicy().hasHeightForWidth())
        self.label_step.setSizePolicy(sizePolicy)
        self.label_step.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_step.setFont(font)
        self.label_step.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_step.setObjectName("label_step")
        self.gridLayout.addWidget(self.label_step, 3, 0, 1, 1)
        self.label_attribute = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_attribute.sizePolicy().hasHeightForWidth())
        self.label_attribute.setSizePolicy(sizePolicy)
        self.label_attribute.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_attribute.setFont(font)
        self.label_attribute.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_attribute.setObjectName("label_attribute")
        self.gridLayout.addWidget(self.label_attribute, 4, 0, 1, 1)
        self.label_settings = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_settings.sizePolicy().hasHeightForWidth())
        self.label_settings.setSizePolicy(sizePolicy)
        self.label_settings.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_settings.setFont(font)
        self.label_settings.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.label_settings.setObjectName("label_settings")
        self.gridLayout.addWidget(self.label_settings, 1, 0, 1, 1)
        self.label_device = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_device.sizePolicy().hasHeightForWidth())
        self.label_device.setSizePolicy(sizePolicy)
        self.label_device.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_device.setFont(font)
        self.label_device.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_device.setObjectName("label_device")
        self.gridLayout.addWidget(self.label_device, 2, 0, 1, 1)
        self.combobox_attribute = QtWidgets.QComboBox(Form)
        self.combobox_attribute.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.combobox_attribute.setObjectName("combobox_attribute")
        self.gridLayout.addWidget(self.combobox_attribute, 4, 1, 1, 1)
        self.combobox_method = QtWidgets.QComboBox(Form)
        self.combobox_method.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.combobox_method.setObjectName("combobox_method")
        self.gridLayout.addWidget(self.combobox_method, 5, 1, 1, 1)
        self.combobox_scanner = QtWidgets.QComboBox(Form)
        self.combobox_scanner.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.combobox_scanner.setObjectName("combobox_scanner")
        self.gridLayout.addWidget(self.combobox_scanner, 6, 1, 1, 1)
        self.button_settings = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.button_settings.sizePolicy().hasHeightForWidth())
        self.button_settings.setSizePolicy(sizePolicy)
        self.button_settings.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/settings.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_settings.setIcon(icon)
        self.button_settings.setIconSize(QtCore.QSize(19, 19))
        self.button_settings.setObjectName("button_settings")
        self.gridLayout.addWidget(self.button_settings, 2, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(1, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 2, 3, 1, 1)
        self.frame_scan_select = QtWidgets.QFrame(Form)
        self.frame_scan_select.setStyleSheet("QFrame {background:#a9c7db}")
        self.frame_scan_select.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_scan_select.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_scan_select.setObjectName("frame_scan_select")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame_scan_select)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_axis_settings = QtWidgets.QLabel(self.frame_scan_select)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_axis_settings.sizePolicy().hasHeightForWidth())
        self.label_axis_settings.setSizePolicy(sizePolicy)
        self.label_axis_settings.setStyleSheet("QLabel {font-weight:bold;padding:5}")
        self.label_axis_settings.setObjectName("label_axis_settings")
        self.horizontalLayout.addWidget(self.label_axis_settings)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.gridLayout.addWidget(self.frame_scan_select, 0, 0, 1, 4)
        spacerItem2 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem2, 7, 0, 1, 3)

        self.retranslateUi(Form)
        self.button_settings.clicked.connect(Form.show_debug_widget)
        self.combobox_device.currentTextChanged['QString'].connect(Form.device_selected)
        self.combobox_step.currentIndexChanged['QString'].connect(Form.step_selected)
        self.combobox_attribute.currentIndexChanged['QString'].connect(Form.attribute_selected)
        self.combobox_method.currentIndexChanged['QString'].connect(Form.method_selected)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.checkbox_defaults.setToolTip(_translate("Form", "Check if You want to set default values for this scan"))
        self.checkbox_defaults.setText(_translate("Form", "Set defaults"))
        self.checkbox_log.setToolTip(_translate("Form", "Check if You want to insert line into log when scan finishes one step"))
        self.checkbox_log.setText(_translate("Form", "Trigger log"))
        self.label_scanner.setText(_translate("Form", "Scanner:"))
        self.label_method.setText(_translate("Form", "Method:"))
        self.label_step.setText(_translate("Form", "Step:"))
        self.label_attribute.setText(_translate("Form", "Attribute:"))
        self.label_settings.setText(_translate("Form", "Settings:"))
        self.label_device.setText(_translate("Form", "Device:"))
        self.button_settings.setToolTip(_translate("Form", "Show device debug widget"))
        self.label_axis_settings.setText(_translate("Form", "Step settings"))
from kamzik3 import resource_kamzik3_rc
